const db = require('../models');
const Utils = require('../utils');

const Sequelize = db.Sequelize;
const Op = Sequelize.Op;

module.exports = class PujaProceduresDBApi {

  static async create(data, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const puja_procedure = await db.puja_procedures.create(
      {
        title: data.title || null,
        description: data.description || null,
        language: data.language || null,
        special_notes: data.special_notes || null,
        // importHash: data.importHash || null,
        // createdById: currentUser.id,
        // updatedById: currentUser.id,
      },
      { transaction },
    );

    return puja_procedure;
  }

  static async findAll(filter, options) {
    var limit = filter.limit || 0;
    var offset = 0;
    const currentPage = +filter.page;

    offset = currentPage * limit;

    var orderBy = null;

    const transaction = (options && options.transaction) || undefined;
    let where = {};
    let include = [];

    if (filter) {
      // if (filter.id) {
      //   where = {
      //     ...where,
      //     ['id']: Utils.uuid(filter.id),
      //   };
      // }

      if (filter.title) {
        where = {
          ...where,
          [Op.and]: Utils.ilike('puja_procedures', 'title', filter.title),
        };
      }

      if (filter.description) {
        where = {
          ...where,
          [Op.and]: Utils.ilike('puja_procedures', 'description', filter.description),
        };
      }

      if (filter.special_notes) {
        where = {
          ...where,
          [Op.and]: Utils.ilike('puja_procedures', 'special_notes', filter.special_notes),
        };
      }

      if (filter.language) {
        where = {
          ...where,
          language: filter.language,
        };
      }
    }

    let { rows, count } = await db.puja_procedures.findAndCountAll({
      where,
      include,
      distinct: true,
      limit: limit ? Number(limit) : undefined,
      offset: offset ? Number(offset) : undefined,
      order:
        filter.field && filter.sort
          ? [[filter.field, filter.sort]]
          : [['updatedAt', 'desc']],
      transaction,
    });

    //    rows = await this._fillWithRelationsAndFilesForRows(
    //      rows,
    //      options,
    //    );

    return { rows, count };
  }

  static async findBy(where, options) {
    const transaction = (options && options.transaction) || undefined;

    const puja_procedures = await db.puja_procedures.findOne(
      { where },
      { transaction },
    );

    if (!puja_procedures) {
      return puja_procedures;
    }

    const output = puja_procedures.get({ plain: true });

    return output;
  }

  static async update(data, id, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const puja_procedures = await db.puja_procedures.findByPk(id, {
      transaction,
    });

    await puja_procedures.update(
      {
        title: data.title || null,
        description: data.description || null,
        language: data.language || null,
        special_notes: data.special_notes || null,
      },
      { transaction },
    );

    return puja_procedures;
  }

  static async remove(id, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const puja_procedures = await db.puja_procedures.findByPk(
      id,
      options,
    );

    await puja_procedures.destroy({
      transaction,
    });
    await Promise.all([
      db.puja_model_procedure_mappings.destroy({
       where:{
        procedure_id: id
       }
     }, {transaction}),
 
      db.booking_procedure_mappings.destroy({
       where:{
        procedure_id: id
       }
     }, {transaction})]);
 
   

    return puja_procedures;
  }
};
