const db = require('../models');
const FileDBApi = require('./file');
const crypto = require('crypto');
const Utils = require('../utils');
const { isObject } = require('lodash');
const { uuid } = require('uuidv4');

const Sequelize = db.Sequelize;
const Op = Sequelize.Op;

module.exports = class PujasDBApi {
  static async create(data, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const pujas = await db.pujas.create(
      {
        //id: data.id || undefined,
        name: data.name || null,
        description: data.description || null,
        language: data.language || null,
        duration_hrs: data.duration_hrs || null,
        importHash: data.importHash || null,
        mode1: data.mode1 || null,
        mode2: data.mode2 || null,
        mode3: data.mode3 || null,
        mode4: data.mode4 || null,
        videos: data.videos || null,
        images: data.images || null,
        createdById: currentUser.id,
        updatedById: currentUser.id,
      },
      { transaction },
    );

    // samagri mappings
    var samagriData = data.samagris;

    const samagriDataToDB = samagriData.map((samagri) => {
      return {
        ...samagri,
        id: uuid(),
        puja_id: pujas.dataValues.id,
      };
    });

    var puja_samagri_mappings;
    const validateSamagri = samagriDataToDB.every(
      (samagri) =>
        samagri.samagri_id !== undefined ||
        (null && samagri.no_of_standard_qty !== undefined) ||
        (null && samagri.isProvidedBy !== undefined) ||
        null,
    );

    if (validateSamagri) {
      puja_samagri_mappings = await db.puja_samagri_mappings.bulkCreate(
        samagriDataToDB,
        {
          transaction,
        },
      );
    } else {
      puja_samagri_mappings = false;
      return {
        message: 'Samagri ID & provided by & quantity  is required',
        statusCode: 500,
      };
    }

    // history mappings
    let historyData = data.history;

    const historyDataToDb = historyData.map((history) => {
      return {
        ...history,
        id: uuid(),
        puja_id: pujas.dataValues.id,
      };
    });
    // historyDataToDb.forEach((history) => {
    //   puja_id: pujas.dataValues.id;
    // });

    var history_mappings;

    const validateHistory = historyDataToDb.every(
      (history) =>
        history.title !== undefined ||
        (null && history.description !== undefined) ||
        null,
    );


    if (validateHistory) {
      history_mappings = await db.puja_history.bulkCreate(historyDataToDb, {
        transaction,
      });
    } else {
      history_mappings = false;

      return {
        message: 'Title & description  is required',
        statusCode: 500,
      };
      // return { message: 'Title & description  is required', statusCode: 500 };
    }

    return {
      statusCode: 200,
      pujas: pujas,
      puja_samagri_mappings: puja_samagri_mappings,
      history_mappings: history_mappings,
    };
  }

  static async update(id, data, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const pujas = await db.pujas.findByPk(id, {
      transaction,
    });

    await pujas.update(
      {
        name: data.name || null,
        description: data.description || null,
        language: data.language || null,
        duration_hrs: data.duration_hrs || null,
        updatedById: currentUser.id,
        mode1: data.mode1 || null,
        mode2: data.mode2 || null,
        mode3: data.mode3 || null,
        mode4: data.mode4 || null,
        videos: data.videos || null,
        images: data.images || null,
      },
      { transaction },
    );

    //  samagri update //

    const newSamagriList = data.samagris.filter(
      (samagri) => samagri.id === undefined,
    );


    const updatingSamagriList = data.samagris.filter(
      (samagri) => samagri.id !== undefined,
    );


    const samagriToDelete = updatingSamagriList.map((samagri) => samagri.id);

    const delteResponse = await db.puja_samagri_mappings.destroy({
      where: { puja_id: id },
      force: true,
    });

    const samagriAddToDB = data.samagris.map((samagri) => {
      return {
        ...samagri,
        id: uuid(),
        puja_id: id,
      };
    });


    const samagriAddedResponse = await db.puja_samagri_mappings.bulkCreate(
      samagriAddToDB,
      {
        transaction,
      },
    );

    // // check whether any new item was adding
    // const validateSamagriData = data.samagris.every(
    //   (obj) => obj.id !== undefined,
    // );

    // if (!validateSamagriData) {
    //   // to get the new items
    //   const newSamagriList = data.samagris.filter(
    //     (samagri) => samagri.id === undefined,
    //   );

    //   // add puja id to new item
    //   const finalSamagriList = newSamagriList.map((object) => {
    //     return { ...object, puja_id: id, id: uuid() };
    //   });

    //   // add the new items to DB
    //   await db.puja_samagri_mappings.bulkCreate(finalSamagriList, {
    //     transaction,
    //   });
    // }

    // // remove new items in update request
    // const samagriAddToDB = data.samagris.filter(
    //   (samagri) => samagri.id !== undefined,
    // );

    // const puja_samagri_mappings = await Promise.all(
    //   samagriAddToDB.map(async (item) => {
    //     return await db.puja_samagri_mappings.update(
    //       {
    //         puja_id: id,
    //         samagri_id: item.samagri_id,
    //         isProvidedBy: item.isProvidedBy,
    //         no_of_standard_qty: item.no_of_standard_qty,
    //       },
    //       { where: { id: item.id } },
    //     );
    //   }),
    // );

    // history update //

    const newHistoryList = data.history.filter(
      (history) => history.id === undefined,
    );

    const updatingHistoryList = data.history.filter(
      (history) => history.id !== undefined,
    );

    const historyToDelete = updatingHistoryList.map((obj) => obj.id);
    await db.puja_history.destroy({
      where: { puja_id: id },
      force: true,
    });

    const historyAddToDB = data.history.map((history) => {
      return {
        ...history,
        id: uuid(),
        puja_id: id,
      };
    });

    await db.puja_history.bulkCreate(historyAddToDB, {
      transaction,
    });

    // // check whether any new history was adding
    // const validateHistoryData = data.history.every(
    //   (obj) => obj.id !== undefined,
    // );

    // if (!validateHistoryData) {
    //   // to get the new history
    //   const newHistoryList = data.history.filter(
    //     (history) => history.id === undefined,
    //   );

    //   // add puja id to new history
    //   const finalHistoryList = newHistoryList.map((object) => {
    //     return { ...object, puja_id: id, id: uuid() };
    //   });

    //   // add the new history to DB
    //   await db.puja_history.bulkCreate(finalHistoryList, {
    //     transaction,
    //   });
    // }

    // // remove new history in update request
    // const historyAddToDB = data.history.filter(
    //   (history) => history.id !== undefined,
    // );

    // const puja_history = await Promise.all(
    //   historyAddToDB.map(async (item) => {
    //     return await db.puja_history.update(
    //       {
    //         puja_id: id,
    //         title: item.title,
    //         description: item.description,
    //       },
    //       { where: { id: item.id } },
    //     );
    //   }),
    // );
    return pujas;
  }

  static async remove(id, options) {
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const pujas = await db.pujas.findByPk(id, options);

    await pujas.update(
      {
        deletedBy: currentUser.id,
      },
      {
        transaction,
      },
    );

    await pujas.destroy({
      transaction,
    });

    return pujas;
  }

  static async findBy(where, options) {
    const transaction = (options && options.transaction) || undefined;
    let include = [db.puja_history, { model: db.puja_samagri_mappings, include: [db.samagri] }];
    const pujas = await db.pujas.findOne({ where, include }, { transaction });

    if (!pujas) {
      return pujas;
    }

    const output = pujas.get({ plain: true });

    return output;
  }

  static async findAll(filter, options) {
    var limit = filter.limit || 0;
    var offset = 0;
    const currentPage = +filter.page;

    offset = currentPage * limit;

    var orderBy = null;

    const transaction = (options && options.transaction) || undefined;
    let where = {};
    let include = [{ model: db.puja_history, required: false }, { model: db.puja_samagri_mappings, required: false }];

    if (filter) {
      if (filter.id) {
        where = {
          ...where,
          ['id']: Utils.uuid(filter.id),
        };
      }

      if (filter.name) {
        where = {
          ...where,
          [Op.and]: Utils.ilike('pujas', 'name', filter.name),
        };
      }

      if (filter.description) {
        where = {
          ...where,
          [Op.and]: Utils.ilike('pujas', 'description', filter.description),
        };
      }

      if (filter.duration_hrsRange) {
        const [start, end] = filter.duration_hrsRange;

        if (start !== undefined && start !== null && start !== '') {
          where = {
            ...where,
            duration_hrs: {
              ...where.duration_hrs,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== '') {
          where = {
            ...where,
            duration_hrs: {
              ...where.duration_hrs,
              [Op.lte]: end,
            },
          };
        }
      }

      if (
        filter.active === true ||
        filter.active === 'true' ||
        filter.active === false ||
        filter.active === 'false'
      ) {
        where = {
          ...where,
          active: filter.active === true || filter.active === 'true',
        };
      }

      if (filter.language) {
        where = {
          ...where,
          language: filter.language,
        };
      }

      if (filter.createdAtRange) {
        const [start, end] = filter.createdAtRange;

        if (start !== undefined && start !== null && start !== '') {
          where = {
            ...where,
            ['createdAt']: {
              ...where.createdAt,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== '') {
          where = {
            ...where,
            ['createdAt']: {
              ...where.createdAt,
              [Op.lte]: end,
            },
          };
        }
      }
    }

    let { rows, count } = await db.pujas.findAndCountAll({
      where,
      include,
      distinct: true,
      limit: limit ? Number(limit) : undefined,
      offset: offset ? Number(offset) : undefined,
      order:
        filter.field && filter.sort
          ? [[filter.field, filter.sort]]
          : [['updatedAt', 'desc']],
      transaction,
    });

    //    rows = await this._fillWithRelationsAndFilesForRows(
    //      rows,
    //      options,
    //    );

    return { rows, count };
  }

  static async findAllAutocomplete(query, limit) {
    let where = {};

    if (query) {
      where = {
        [Op.or]: [
          { ['id']: Utils.uuid(query) },
          Utils.ilike('pujas', 'id', query),
        ],
      };
    }

    const records = await db.pujas.findAll({
      attributes: ['id', 'id'],
      where,
      limit: limit ? Number(limit) : undefined,
      orderBy: [['id', 'ASC']],
    });

    return records.map((record) => ({
      id: record.id,
      label: record.id,
    }));
  }
};
