'use strict';

const { uuid } = require('uuidv4');
const fs = require('fs');
const path = require('path');
const Sequelize = require('sequelize');
const basename = path.basename(__filename);
const env = process.env.NODE_ENV || 'development';
const config = require('../db.config')[env];
const db = {};

let sequelize;
console.log(env);
if (config.use_env_variable) {
  sequelize = new Sequelize(process.env[config.use_env_variable], config);
} else {
  sequelize = new Sequelize(
    config.database,
    config.username,
    config.password,
    config,
    {
      host: config.host,
      dialect: "mysql",
      pool: {
        max: 20,
        min: 0,
        acquire: 60000,
        idle: 10000
      }
    }
  );
}

fs.readdirSync(__dirname)
  .filter((file) => {
    return (
      file.indexOf('.') !== 0 && file !== basename && file.slice(-3) === '.js'
    );
  })
  .forEach((file) => {
    const model = require(path.join(__dirname, file))(
      sequelize,
      Sequelize.DataTypes,
    );
    db[model.name] = model;
  });

Object.keys(db).forEach((modelName) => {
  if (db[modelName].associate) {
    db[modelName].associate(db);
  }
});

db.sequelize = sequelize;
db.Sequelize = Sequelize;

db.puja_samagri_mappings.beforeCreate((model, _) => {
  return (model.id = uuid());
});

db.puja_history.beforeCreate((user, _) => {
  return (user.id = uuid());
});

db.samagri.beforeCreate((samagri, _) => {
  return (samagri.id = uuid());
});

db.puja_model_procedure_mappings.beforeCreate((procedure, _) => {
  return (procedure.id = uuid());
});

module.exports = db;
